
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#pragma once

#include "resource.h"       // main symbols
#include "ExcelClone.h"
#include "Workbooks.h"

#include "DispatchHook.h"


/*
	CApplication class provides functionality of the MS Excel Application object by using Open Office.
	It implements IApplication interface of the MS Excel.
*/
class ATL_NO_VTABLE CApplication : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CApplication, &CLSID_Application>,
    public CDispatchHook< IDispatchImpl<IApplication, &IID_IApplication, &LIBID_ExcelCloneLib, 1, 0> >
{
public:
	CApplication();
	~CApplication();

    DECLARE_CLASSFACTORY_SINGLETON(CApplication)

    DECLARE_REGISTRY_RESOURCEID(IDR_EXCELAPPLICATION)

    DECLARE_NOT_AGGREGATABLE(CApplication)

    BEGIN_COM_MAP(CApplication)
	    COM_INTERFACE_ENTRY(IApplication)
	    COM_INTERFACE_ENTRY(IDispatch)
    END_COM_MAP()

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
	}

public: 

    STDMETHOD(put_UserControl)(VARIANT_BOOL vbUserControl);		
	// True if the application is visible or if it was created or started by the user. 
	// False if you created or started the application programmatically by using the 
	// CreateObject or GetObject functions, and the application is hidden. 

    STDMETHOD(put_DisplayAlerts)(VARIANT_BOOL vbDisplayAlerts);
	// True if Microsoft Excel displays certain alerts and messages while a macro is running. 
    
    STDMETHOD(put_WindowState)(XlWindowState State);
	// Returns or sets the state of the window. Read/write XlWindowState.
	// XlWindowState can be one of these XlWindowState constants. 
	// xlMaximized 
	// xlNormal 
	// xlMinimized 

    STDMETHOD(put_Visible)(VARIANT_BOOL vbFlag);
	// Determines whether the object is visible. 

    STDMETHOD(get_Workbooks)(IWorkbooks **ppWorkbooks);
	// Returns a Workbooks collection that represents all the open workbooks.

    STDMETHOD(get_Sheets)(ISheets **ppSheets);
	// Returns a Sheets collection that represents all the sheets in the active workbook, 
	// for an Application object. Returns a Sheets collection that represents all the sheets 
	// in the specified workbook, for a Workbook object.

	STDMETHOD(get_Cells)(IRange **ppRange);
	// Returns a Range object that represents all the cells on the active worksheet. If the 
	// active document isnt a worksheet, this property fails.

	STDMETHOD(get_ActiveSheet)(IDispatch** ppActiveSheet);

    STDMETHOD(get_Version)(long Lcid, BSTR *pVersion);
        
    STDMETHOD(ConvertFormula)( 
        VARIANT          Formula,
        XlReferenceStyle FromReferenceStyle,
        VARIANT          ToReferenceStyle,
        VARIANT          ToAbsolute,
        VARIANT          RelativeTo,
        long             Lcid,
        VARIANT         *pResult);
    
    STDMETHOD(Quit)();

    STDMETHOD(get_ActiveCell) (IRange** ppRange);

    STDMETHOD(get_ActiveWorkbook)(IWorkbook **ppWorkbook);

public:
    CWorkbooksObject m_Workbooks; 
	// Open Office data (IDispatch)

	friend class CWorkbooks; 
	friend class CWorkbook;

public:
	// Retrieve (IDispatch) OpenOffice's com.sun.star.beans.PropertyValue object
	int GetDispatchPropertyValue(IDispatch** pIDispatch); 


    std::wstring generateFileName();        
    static IDispatch  *s_pdOOApp;	// (IDispatch) OpenOffice application
	static IDispatch  *s_pdDesktop; // (IDispatch) OpenOffice desktop

};

OBJECT_ENTRY_AUTO(__uuidof(Application), CApplication)
