
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#pragma once

#include "ExcelClone.h"
#include "DispatchHook.h"

struct CELL_COORD
{
    long x, y;

    CELL_COORD(long _x, long _y): x(_x), y(_y)
    {
    }
};


/*
	CRange class represents a cell, a row, a column, a selection of cells containing 
	one or more contiguous blocks of cells, or a 3-D range. It implements IRange COM interface of the MS Excel.


*/

class ATL_NO_VTABLE CRange :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CDispatchHook< IDispatchImpl<IRange, &IID_IRange, &LIBID_ExcelCloneLib, 1, 0> >
{
public:
    CRange();
    ~CRange();

    HRESULT Initialize(CRange *pParentRange, const CELL_COORD &topLeft, const CELL_COORD &bottomRight);
	HRESULT Initialize(IDispatch *pThisDisp);


    BEGIN_COM_MAP(CRange)
        COM_INTERFACE_ENTRY(IRange)
        COM_INTERFACE_ENTRY(IDispatch)
    END_COM_MAP()

public:
    STDMETHOD(get__Default)(VARIANT varRowIndex, VARIANT varColumnIndex, IDispatch **ppObject);
    // Get specified range, whic is be sub range of this range

	STDMETHOD(get_ColumnWidth)(long *pnColumnWidth);
    STDMETHOD(put_ColumnWidth)(long nColumnWidth);
	// Returns or sets the width of all columns in the specified range (params specified in chars). 

    STDMETHOD(get_Font)(IXLFont **ppFont);
	// Returns a Font object that represents the font of the specified object.
    
    STDMETHOD(get_Value)(VARIANT varRangeValueDataType, VARIANT *pvarValue);
    STDMETHOD(put_Value)(VARIANT varRangeValueDataType, VARIANT varValue);
    // Returns or sets the value of the specified range

    STDMETHOD(Select)(VARIANT *pvarResult);
	// To select a cell or a range of cells, use the Select method

    STDMETHOD(NoteText)( 
        VARIANT  Text,
        VARIANT  Start,
        VARIANT  Length,
        BSTR    *pText
        );

    STDMETHOD_(VARIANT, ClearContents) ();

    STDMETHOD_(long, get_Column) ();

    STDMETHOD_(long, get_Row) ();

protected:
	IDispatch  *m_pdRange; // OO range
    friend class CXLFont;
public:
    HRESULT GetRangeAddress(long &lLeft, long &lTop, long &lRight, long &lBottom);
};

typedef CComInternalObject<CRange> CRangeIntObject;
typedef CComObject<CRange> CRangeObject;
typedef CComPtr<CRangeObject> CRangeObjectPtr;
